/***********************************/
/* NosDAC: Module to support       */
/* Sserial 16-bit Non-oversampling */
/* digital audio DAC               */
/* using SPI and TIMER2 module     */
/* of ATmega168                    */
/*                                 */
/* 2009/6/26 created by PCM1723    */
/* 2009/7/01 'PD3' -> 'PIND3'      */
/*           for ATmega168P        */
/***********************************/
//
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <Wiring.h>
#include "NosDAC.h"
//
// DAC buffer to explode one int16_t into two uint8_t
//
static volatile union
{
  int16_t w;
  uint8_t b[2];
} dac_save; // static volatile union dac_save{}
//
// Timer2 COMPare match A interrupt service routine
// xmit upper byte to 16-bit DAC via 8-bit SPI
//
ISR(TIMER2_COMPA_vect) 
{
    SPSR;                // read SPSR to clear SPIF
    if (_BV(PIND3) & PIND) { // get LRCK level
      dac_save.w = dac_l;    // save DAC data for later use
      fsflag  = 1;  // notify fs update timing to main loop
    } // if ()
    SPDR  = dac_save.b[1];  // xmit high byte
    SPCR |= (_BV(SPIE)); // enable SPI interrupt to xmit low byte
} // ISR(TIMER2_COMPA_vect) 
//
// SPI Serial Transfer Complete interrupt service routine
// xmit lower byte to 16-bit DAC via 8-bit SPI
//
ISR(SPI_STC_vect) {
  SPDR  = dac_save.b[0]; // low byte
  SPCR &= (~_BV(SPIE)); // disable SPI interrupt
} // ISR(SPI_STC_vect)
//
// SPI setup to support Non-oversampling DAC
//
void NosDAC_setup(int8_t DAC_mode)
{
// modify Timer2 setting for LRCK generateion
  analogWrite(LRCK_PIN,0); // enable LRCK output
  TCCR2B  = 0x02; // 1/8 clk (2 MHz)
  TCCR2A  = 0x12; // CTC mode
                  // OC2A disconn, OC2B toggle
  OCR2A   = 0x3F; // MAX = 0x3F (2 MHz / 64 / 2 = 31.25 kHz)
  TIMSK2 |= _BV(OCIE2A);  // enable TIMER2 OCF2A interrupt 
// SPI setup for serial 16 bit digital audio DAC 
// set SCK, MOSI to OUTPUT
  pinMode(SCK_PIN,  OUTPUT);
  pinMode(MOSI_PIN, OUTPUT);
// to assure MASTER operation, set SS to OUTPUT  
  pinMode(SS_PIN,   OUTPUT);
// MISO forced to INPUT by MASTER mode, enable pullup
  pinMode(MISO_PIN, INPUT);
  digitalWrite(MISO_PIN, HIGH); // enable pullup at MISO
// enable SPI, fosc/16, Master, CPHA=0, DORD=0, CPOL=0
  SPCR   = (_BV(SPE) | _BV(MSTR) | _BV(SPR0)); 
} // void NosDAC_setup()
